/*

This file contains Interrupt Service Routine for DMA2(NFC) and NFC Error
DMA2 is used for Page Read Operation and NFC Error is used for Page Write Completion,
Buffer Overflow and for Rising Edge on ND_RB signal

*/


#include "Extern_Declaration.h"

unsigned int Stored_ECC[Flash_Spare_Area];
unsigned int Calc_ECC[Flash_Spare_Area];

unsigned short Write_ECC_Computed_Flag = 0;
unsigned short Read_ECC_Computed_Flag = 0; 

unsigned short Temp_Write_ECC[Flash_Spare_Area];// This is for temp storage of ECC during writes. 

short m, n = 0;
short NBUSY_Flag = 0;

/**********************************************************************/

EX_INTERRUPT_HANDLER(NFC_ISR)
{
		//This is for Page Read
		
		//wait for DMA done bit to set
	
		while(!(*pDMA2_IRQ_STATUS & DMA_DONE));
		*pDMA2_IRQ_STATUS |= DMA_DONE;
		ssync();
		
		
		//Copy the ECC0 and 1 values
					
		Calc_ECC[m++] = *pNFC_ECC0; 
		Calc_ECC[m++] = *pNFC_ECC1;
			
			
		if(NFC_PAGE_SIZE == 512)
		{
			//Copy the ECC2 and 3 values
		
			Calc_ECC[m++] = *pNFC_ECC2; 
			Calc_ECC[m++] = *pNFC_ECC3;
		
		}
						
		Read_ECC_Computed_Flag = 1;			
	
}
		
/**********************************************************************/

short temp_read;

EX_INTERRUPT_HANDLER(NFC_IRQSTAT_ISR)
{
	
	/* 
	
	check if this interrupt is because of NBUSYIRQ or WR_DONE
	WR_DONE is for Page Write
	NBUSYIRQ for rising edge on /ND_RD 
	
	*/
	
	if(*pNFC_IRQSTAT & WR_DONE)
	{
		// W1C WR_DONE
		*pNFC_IRQSTAT |= WR_DONE;
		ssync();
		
		//Copy the ECC0 and 1 values
						
		Temp_Write_ECC[n++] = *pNFC_ECC0; 
		Temp_Write_ECC[n++] = *pNFC_ECC1;
		 
		
		if(NFC_PAGE_SIZE == 512)
		{
			//Copy the ECC2 and 3 values
			
			
			Temp_Write_ECC[n++] = *pNFC_ECC2; 
			Temp_Write_ECC[n++] = *pNFC_ECC3;
			 
		
		}
		
		Write_ECC_Computed_Flag = 1;	
	}
			
	
	else if(*pNFC_IRQSTAT & NBUSYIRQ)
	
	{
		// Set Flag1, which indicates NBUSYIRQ is cleared
		NBUSY_Flag = 1;
		//Interrupt is due to NBUSYIRQ. W1C
		*pNFC_IRQSTAT |= NBUSYIRQ;
		ssync();
		
		
	}
	
	else if(*pNFC_IRQSTAT & WB_OVF)
	
	{
		//Interrupt is due to Buffer Overflow. W1C
		printf("Error....Buffer Overflow...\n");
		*pNFC_IRQSTAT |= WB_OVF;
		ssync();
		
	}
	
	

}

/**********************************************************************/
