/*

This function reads a Page of a Block. Once the Page read is completed
Error Analysis is performed to check for any errors occured

Parameter to be passed: Page No, BlockNo, Pointer where data to be stored
Returns: None

*/


#include "Extern_Declaration.h"

extern int k;	// used in Error Analysis
extern short m;	// used in ISR
extern unsigned int Stored_ECC[];
extern short Read_ECC_Computed_Flag; 
extern short ECC_Error_Flag;

/**********************************************************************/

void PageRead(unsigned int Rpage, unsigned int Rblock, unsigned char *Read_Buf)
{
	
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	// Page Read Command:
	
	*pNFC_CMD =  PageReadCmd[0];
	 
	/*  
	
	Set up the address cycles depending on Page and Block number passed:
	
	Column: A0-A11
	Page: 	A12-A17
	Block: 	A18-A28
	
	
	1st Address Cycle: A0-A7
	2nd Address Cycle: A8-A11
	3rd Address Cycle: A12-A19
	4th Address Cycle: A20-A27
	5th Address Cycle: A28
	
	*/
	
	short temp_Page = Rpage;
	short temp_Block = Rblock;
	
	//Column
	*pNFC_ADDR = 0;
	*pNFC_ADDR = 0;
	
	//Page
	
	temp_Page = Rblock & 3;		// Get bits A18 and A19 from Block address
	temp_Page = temp_Page << 6;	// Shift to append with Page Address
	
	Rpage = Rpage | temp_Page;
	
	// First 2 bits of Block and 6 bits of Page are sent
	*pNFC_ADDR = Rpage;	
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));	
	
	// Next 8 bits of Block address. i.e till 1023. 
			
	Rblock = temp_Block;
	Rblock = Rblock & 0x3FC;	// Mask to get A20-A27	
	Rblock = Rblock >> 2;		// Because A18 and A19 are already sent out
	
	*pNFC_ADDR = Rblock;
	
		
	// Block address greater than 1023
	
	Rblock = temp_Block;
	Rblock = Rblock & 0xFC00;	// Mask to get A28-A30
	Rblock = Rblock >> 10;		// Because A18-A27 are already sent out
	
	*pNFC_ADDR = Rblock;
	
	// Next command
	
	*pNFC_CMD =  PageReadCmd[1];
	
	//Wait till NBUSY_Flag sets here.
	
	while(!NBUSY_Flag); 
	NBUSY_Flag = 0;
	
	/* 
	
	Configure the DMA22. 
	This is shared with SDH. So clear PeriMUX register of DMAC1
	
	*/

	*pDMAC1_PERIMUX = 0;
		
	*pDMA22_CONFIG = DI_EN | WDSIZE & (8) | WNR ;
	*pDMA22_START_ADDR = Read_Buf;
	*pDMA22_X_COUNT = NFC_PAGE_SIZE/4;  
	*pDMA22_X_MODIFY = 4;				//fifo 32 bit
	*pDMA22_Y_COUNT = 0;
	*pDMA22_Y_MODIFY = 0;
	
	*pDMA22_CONFIG |= DMAEN ;
		
	
	for(i = 0; i<(Flash_Page_Size/NFC_PAGE_SIZE); i++)
	{
		// DMA is configured. Issue Page Read
	
		*pNFC_RST = ECC_RST;
		*pNFC_PGCTL = PG_RD_START;

		// Wait for ECC to be computed.
		while(!(Read_ECC_Computed_Flag));
		Read_ECC_Computed_Flag = 0;
	
		// Disable the DMA
		
		*pDMA22_CONFIG &= 0xFFFE;						
		
		// Re-config start address for next DMA transfer 
				
		*pDMA22_START_ADDR = *pDMA22_CURR_ADDR;
			
		// Re-enable the DMA
			
		*pDMA22_CONFIG |= 0x0001;
	}
	
	/* 
	
	Read the Spare Area where the ECC is stored. Put that into 
	Stored_ECC buffer. Since ECC register is 16 bits wide, but Flash Data width
	is 8 bits, first we read two 8 bits to make it to read the ECC register.

	Actual loop count is (2*2*Flash_Page_Size/256) (22 bit ECC generated for every 256 
	which is 4 bytes/256 page )

	Since we reading 2 byte for one iteration, loop count is taken as (2*Flash_Page_Size/256)
	
	
	*/

	int temp1, temp2; 
		
	for(i=0; i<(2*Flash_Page_Size/256); i++)
	{
		while(!(*pNFC_STAT & WB_EMPTY));
		
		// ECC register is read as two bytes
		
		*pNFC_DATA_RD = 0;
		while(!(*pNFC_IRQSTAT & RD_RDY));
		temp1 = *pNFC_READ;
		*pNFC_IRQSTAT |= RD_RDY;
		ssync();
					
			
		*pNFC_DATA_RD = 0;
		while(!(*pNFC_IRQSTAT & RD_RDY));
		temp2 = *pNFC_READ;
		*pNFC_IRQSTAT |= RD_RDY;
		ssync();
		
		temp2 <<= 8; 
		temp1 = temp1 | temp2;	
		
		Stored_ECC[i] = temp1;
	}
			
	/* 
	
	Calculate the Syndrome and see if there is any error.		
	The ECC is calculated for every 256 bytes of the page.
	So call the Error_Analysis 8 times (for 2K page Flash)
	
		
	*/

	
	for(i = 0; i <(Flash_Page_Size/256); i++)
	{
		Error_Analysis();
	
		
	}
	
	if(Flag1 == 1)
	{
		printf("Page Read Completed \n");
	}
		
	// Clear the variables used in other functions
	
	k, m = 0;
	
	
}

	
/****************************************************************************/

