/*

This function reads an byte from a page using Random Data Output command

Parameter to be passed: ByteNo, PageNo, BlockNo, data to be written

Returns: None

*/

#include "Extern_Declaration.h"

/***************************************************************************************/

void RandomDataOutput(unsigned int RObyte, unsigned int ROpage, unsigned int ROblock, unsigned char DataOut)
{
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));
	
	// Page Read Command:
	
	*pNFC_CMD =  PageReadCmd[0];
	 
	/*  
	
	Set up the address cycles depending on Page and Block number passed:
	
	Column: A0-A11
	Page: 	A12-A17
	Block: 	A18-A28
	
	
	1st Address Cycle: A0-A7
	2nd Address Cycle: A8-A11
	3rd Address Cycle: A12-A19
	4th Address Cycle: A20-A27
	5th Address Cycle: A28
	
	*/
	
	short temp_Page = ROpage;
	short temp_Block = ROblock;
	
	//Column
	*pNFC_ADDR = 0;
	*pNFC_ADDR = 0;
	
	//Page
	
	temp_Page = ROblock & 3;		// Get bits A18 and A19 from Block address
	temp_Page = temp_Page << 6;	// Shift to append with Page Address
	
	ROpage = ROpage | temp_Page;
	
	// First 2 bits of Block and 6 bits of Page are sent
	*pNFC_ADDR = ROpage;	
	
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));	
	
	// Next 8 bits of Block address. i.e till 1023. 
			
	ROblock = temp_Block;
	ROblock = ROblock & 0x3FC;	// Mask to get A20-A27	
	ROblock = ROblock >> 2;		// Because A18 and A19 are already sent out
	
	*pNFC_ADDR = ROblock;
	
		
	// Block address greater than 1023
	
	ROblock = temp_Block;
	ROblock = ROblock & 0xFC00;	// Mask to get A28-A30
	ROblock = ROblock >> 10;		// Because A18-A27 are already sent out
	
	*pNFC_ADDR = ROblock;
	
	// Next command
	
	*pNFC_CMD =  PageReadCmd[1];
	
	//Wait till NBUSY_Flag sets here.
	
	while(!NBUSY_Flag); 
	NBUSY_Flag = 0;
	
		
	// Issue Random Data Output command
	*pNFC_CMD = RandomDataOutputCmd[0];
		
		
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));	
		
		
	// Now issue Address cycle to change the Column address
		
	*pNFC_ADDR = RObyte;		//A0-A7
	*pNFC_ADDR = RObyte >> 8;	//A8-A11
		
	// Issue Random Data Output command
	*pNFC_CMD = RandomDataOutputCmd[1];
				
	
	// Read the byte here
	// Only one data is readin this example. Once the column address is changed
	// user can read the remaining bytes of a page. Ensure that Read request is issued each time
	
			
	// Wait till write buffer is empty:
	
	while(!(*pNFC_STAT & WB_EMPTY));	
		
	
	*pNFC_DATA_RD = 0;
	while(!(*pNFC_IRQSTAT & RD_RDY));
	DataOut = *pNFC_READ;
	*pNFC_IRQSTAT |= RD_RDY;
	ssync();
	
	printf("Random Data Output is completed. Data read is %x \n", DataOut);
	
}	
	
	
	
	
